@@ -76,29 +76,43 @@ typedef struct {
   MetaXWaylandManager *manager;
   MetaWindow *window;
   guint32 surface_id;
-  guint idle_id;
+  guint later_id;
 } AssociateWindowWithSurfaceOp;
 
+static void associate_window_with_surface_window_unmanaged (MetaWindow                   *window,
+                                                            AssociateWindowWithSurfaceOp *op);
 static void
-associate_window_with_surface_window_destroyed (gpointer  user_data,
-                                                GObject  *obj)
+associate_window_with_surface_op_free (AssociateWindowWithSurfaceOp *op)
 {
-  AssociateWindowWithSurfaceOp *op = user_data;
-  g_source_remove (op->idle_id);
+  if (op->later_id != 0)
+    meta_later_remove (op->later_id);
+  g_signal_handlers_disconnect_by_func (op->window,
+                                        (gpointer) associate_window_with_surface_window_unmanaged,
+                                        op);
   g_free (op);
 }
 
+static void
+associate_window_with_surface_window_unmanaged (MetaWindow                   *window,
+                                                AssociateWindowWithSurfaceOp *op)
+{
+  associate_window_with_surface_op_free (op);
+}
+
 static gboolean
-associate_window_with_surface_idle (gpointer user_data)
+associate_window_with_surface_later (gpointer user_data)
 {
   AssociateWindowWithSurfaceOp *op = user_data;
+
+  op->later_id = 0;
+
   if (!associate_window_with_surface_id (op->manager, op->window, op->surface_id))
     {
       /* Not here? Oh well... nothing we can do */
       g_warning ("Unknown surface ID %d (from window %s)", op->surface_id, op->window->desc);
     }
-  g_object_weak_unref (G_OBJECT (op->window), associate_window_with_surface_window_destroyed, op);
-  g_free (op);
+
+  associate_window_with_surface_op_free (op);
 
   return G_SOURCE_REMOVE;
 }
@@ -113,17 +127,20 @@ meta_xwayland_handle_wl_surface_id (MetaWindow *window,
   if (!associate_window_with_surface_id (manager, window, surface_id))
     {
       /* No surface ID yet... it should arrive after the next
-       * iteration through the loop, so queue an idle and see
+       * iteration through the loop, so queue a later and see
        * what happens.
        */
       AssociateWindowWithSurfaceOp *op = g_new0 (AssociateWindowWithSurfaceOp, 1);
       op->manager = manager;
       op->window = window;
       op->surface_id = surface_id;
-      op->idle_id = g_idle_add (associate_window_with_surface_idle, op);
-      g_source_set_name_by_id (op->idle_id, "[mutter] associate_window_with_surface_idle");
+      op->later_id = meta_later_add (META_LATER_BEFORE_REDRAW,
+                                     associate_window_with_surface_later,
+                                     op,
+                                     NULL);
 
-      g_object_weak_ref (G_OBJECT (op->window), associate_window_with_surface_window_destroyed, op);
+      g_signal_connect (op->window, "unmanaged",
+                        G_CALLBACK (associate_window_with_surface_window_unmanaged), op);
     }
 }
 
